from .ast_builder import AstBuilder


class TokenFormatterBuilder(AstBuilder):
    def __init__(self):
        self.reset()

    def reset(self):
        self._tokens = []

    def build(self, token):
        self._tokens.append(token)

    def start_rule(self, rule_type):
        pass

    def end_rule(self, rule_type):
        pass

    def get_result(self):
        return '\n'.join([self._format_token(token) for token in self._tokens])

    def _format_token(self, token):
        if token.eof():
            return "EOF"

        return ''.join([
            '(',
            str(token.location['line']), ':', str(token.location['column']),
            ')',
            token.matched_type,
            ':',
            (token.matched_keyword if token.matched_keyword else ''),
            '/',
            (token.matched_text if token.matched_text else ""),
            '/',
            ','.join([str(item['column']) + ':' + item['text'] for item in token.matched_items])
        ])



from gherkin.parser import Parser
from gherkin.pickles.compiler import compile
from gherkin.errors import ParserError, CompositeParserException

def add_errors(events, errors, uri):
    for error in errors:
        events.append({
            'type': 'attachment',
            'source': {
                'uri': uri,
                'start': {
                    'line': error.location['line'],
                    'column': error.location['column']
                }
            },
            'data': str(error),
            'media': {
                'encoding': 'utf-8',
                'type': 'text/vnd.cucumber.stacktrace+plain'
            }
        })

class GherkinEvents:
    def __init__(self, options):
        self.options = options
        self.parser = Parser()

    def enum(self, source_event):
        uri = source_event['uri']
        source = source_event['data']

        events = []

        try:
            gherkin_document = self.parser.parse(source)

            if (self.options.print_source):
                events.append(source_event)

            if (self.options.print_ast):
                events.append({
                    'type': 'gherkin-document',
                    'uri': uri,
                    'document': gherkin_document
                })

            if (self.options.print_pickles):
                pickles = compile(gherkin_document)
                for pickle in pickles:
                    events.append({
                        'type': 'pickle',
                        'uri': uri,
                        'pickle': pickle
                    })
        except CompositeParserException as e:
            add_errors(events, e.errors, uri)
        except ParserError as e:
            add_errors(events, [e], uri)

        return events



# This file is generated. Do not edit! Edit gherkin-python.razor instead.
import sys
from collections import deque
from .ast_builder import AstBuilder
from .token_matcher import TokenMatcher
from .token_scanner import TokenScanner
from .errors import UnexpectedEOFException, UnexpectedTokenException, ParserException, CompositeParserException

RULE_TYPE = [
    'None',
    '_EOF',  # #EOF
    '_Empty',  # #Empty
    '_Comment',  # #Comment
    '_TagLine',  # #TagLine
    '_FeatureLine',  # #FeatureLine
    '_BackgroundLine',  # #BackgroundLine
    '_ScenarioLine',  # #ScenarioLine
    '_ScenarioOutlineLine',  # #ScenarioOutlineLine
    '_ExamplesLine',  # #ExamplesLine
    '_StepLine',  # #StepLine
    '_DocStringSeparator',  # #DocStringSeparator
    '_TableRow',  # #TableRow
    '_Language',  # #Language
    '_Other',  # #Other
    'GherkinDocument',  # GherkinDocument! := Feature?
    'Feature',  # Feature! := Feature_Header Background? Scenario_Definition*
    'Feature_Header',  # Feature_Header! := #Language? Tags? #FeatureLine Feature_Description
    'Background',  # Background! := #BackgroundLine Background_Description Scenario_Step*
    'Scenario_Definition',  # Scenario_Definition! := Tags? (Scenario | ScenarioOutline)
    'Scenario',  # Scenario! := #ScenarioLine Scenario_Description Scenario_Step*
    'ScenarioOutline',  # ScenarioOutline! := #ScenarioOutlineLine ScenarioOutline_Description ScenarioOutline_Step* Examples_Definition*
    'Examples_Definition',  # Examples_Definition! [#Empty|#Comment|#TagLine-&gt;#ExamplesLine] := Tags? Examples
    'Examples',  # Examples! := #ExamplesLine Examples_Description Examples_Table?
    'Examples_Table',  # Examples_Table! := #TableRow #TableRow*
    'Scenario_Step',  # Scenario_Step := Step
    'ScenarioOutline_Step',  # ScenarioOutline_Step := Step
    'Step',  # Step! := #StepLine Step_Arg?
    'Step_Arg',  # Step_Arg := (DataTable | DocString)
    'DataTable',  # DataTable! := #TableRow+
    'DocString',  # DocString! := #DocStringSeparator #Other* #DocStringSeparator
    'Tags',  # Tags! := #TagLine+
    'Feature_Description',  # Feature_Description := Description_Helper
    'Background_Description',  # Background_Description := Description_Helper
    'Scenario_Description',  # Scenario_Description := Description_Helper
    'ScenarioOutline_Description',  # ScenarioOutline_Description := Description_Helper
    'Examples_Description',  # Examples_Description := Description_Helper
    'Description_Helper',  # Description_Helper := #Empty* Description? #Comment*
    'Description',  # Description! := #Other+
]


class ParserContext(object):
    def __init__(self, token_scanner, token_matcher, token_queue, errors):
        self.token_scanner = token_scanner
        self.token_matcher = token_matcher
        self.token_queue = token_queue
        self.errors = errors


class Parser(object):
    def __init__(self, ast_builder=AstBuilder()):
        self.ast_builder = ast_builder
        self.stop_at_first_error = False

    def parse(self, token_scanner_or_str, token_matcher=TokenMatcher()):
        if sys.version_info < (3, 0):
            token_scanner = TokenScanner(token_scanner_or_str) if isinstance(token_scanner_or_str, basestring) else token_scanner_or_str
        else:
            token_scanner = TokenScanner(token_scanner_or_str) if isinstance(token_scanner_or_str, str) else token_scanner_or_str
        self.ast_builder.reset()
        token_matcher.reset()
        context = ParserContext(
            token_scanner,
            token_matcher,
            deque(),
            [])

        self.start_rule(context, 'GherkinDocument')
        state = 0
        token = None
        while True:
            token = self.read_token(context)
            state = self.match_token(state, token, context)
            if token.eof():
                break

        self.end_rule(context, 'GherkinDocument')

        if context.errors:
            raise CompositeParserException(context.errors)

        return self.get_result()

    def build(self, context, token):
        self.handle_ast_error(context, token, self.ast_builder.build)

    def add_error(self, context, error):
        context.errors.append(error)
        if len(context.errors) > 10:
            raise CompositeParserException(context.errors)

    def start_rule(self, context, rule_type):
        self.handle_ast_error(context, rule_type, self.ast_builder.start_rule)

    def end_rule(self, context, rule_type):
        self.handle_ast_error(context, rule_type, self.ast_builder.end_rule)

    def get_result(self):
        return self.ast_builder.get_result()

    def read_token(self, context):
        if context.token_queue:
            return context.token_queue.popleft()
        else:
            return context.token_scanner.read()

    def match_EOF(self, context, token):
        return self.handle_external_error(context, False, token, context.token_matcher.match_EOF)

    def match_Empty(self, context, token):
        if token.eof():
            return False
        return self.handle_external_error(context, False, token, context.token_matcher.match_Empty)

    def match_Comment(self, context, token):
        if token.eof():
            return False
        return self.handle_external_error(context, False, token, context.token_matcher.match_Comment)

    def match_TagLine(self, context, token):
        if token.eof():
            return False
        return self.handle_external_error(context, False, token, context.token_matcher.match_TagLine)

    def match_FeatureLine(self, context, token):
        if token.eof():
            return False
        return self.handle_external_error(context, False, token, context.token_matcher.match_FeatureLine)

    def match_BackgroundLine(self, context, token):
        if token.eof():
            return False
        return self.handle_external_error(context, False, token, context.token_matcher.match_BackgroundLine)

    def match_ScenarioLine(self, context, token):
        if token.eof():
            return False
        return self.handle_external_error(context, False, token, context.token_matcher.match_ScenarioLine)

    def match_ScenarioOutlineLine(self, context, token):
        if token.eof():
            return False
        return self.handle_external_error(context, False, token, context.token_matcher.match_ScenarioOutlineLine)

    def match_ExamplesLine(self, context, token):
        if token.eof():
            return False
        return self.handle_external_error(context, False, token, context.token_matcher.match_ExamplesLine)

    def match_StepLine(self, context, token):
        if token.eof():
            return False
        return self.handle_external_error(context, False, token, context.token_matcher.match_StepLine)

    def match_DocStringSeparator(self, context, token):
        if token.eof():
            return False
        return self.handle_external_error(context, False, token, context.token_matcher.match_DocStringSeparator)

    def match_TableRow(self, context, token):
        if token.eof():
            return False
        return self.handle_external_error(context, False, token, context.token_matcher.match_TableRow)

    def match_Language(self, context, token):
        if token.eof():
            return False
        return self.handle_external_error(context, False, token, context.token_matcher.match_Language)

    def match_Other(self, context, token):
        if token.eof():
            return False
        return self.handle_external_error(context, False, token, context.token_matcher.match_Other)

    def match_token(self, state, token, context):
        state_map = {
            0: self.match_token_at_0,
            1: self.match_token_at_1,
            2: self.match_token_at_2,
            3: self.match_token_at_3,
            4: self.match_token_at_4,
            5: self.match_token_at_5,
            6: self.match_token_at_6,
            7: self.match_token_at_7,
            8: self.match_token_at_8,
            9: self.match_token_at_9,
            10: self.match_token_at_10,
            11: self.match_token_at_11,
            12: self.match_token_at_12,
            13: self.match_token_at_13,
            14: self.match_token_at_14,
            15: self.match_token_at_15,
            16: self.match_token_at_16,
            17: self.match_token_at_17,
            18: self.match_token_at_18,
            19: self.match_token_at_19,
            20: self.match_token_at_20,
            21: self.match_token_at_21,
            22: self.match_token_at_22,
            23: self.match_token_at_23,
            24: self.match_token_at_24,
            25: self.match_token_at_25,
            26: self.match_token_at_26,
            28: self.match_token_at_28,
            29: self.match_token_at_29,
            30: self.match_token_at_30,
            31: self.match_token_at_31,
            32: self.match_token_at_32,
            33: self.match_token_at_33,
        }
        if state in state_map:
            return state_map[state](token, context)
        else:
            raise RuntimeError("Unknown state: " + str(state))

    # Start
    def match_token_at_0(self, token, context):
        if self.match_EOF(context, token):
                self.build(context, token)
                return 27
        if self.match_Language(context, token):
                self.start_rule(context, 'Feature')
                self.start_rule(context, 'Feature_Header')
                self.build(context, token)
                return 1
        if self.match_TagLine(context, token):
                self.start_rule(context, 'Feature')
                self.start_rule(context, 'Feature_Header')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 2
        if self.match_FeatureLine(context, token):
                self.start_rule(context, 'Feature')
                self.start_rule(context, 'Feature_Header')
                self.build(context, token)
                return 3
        if self.match_Comment(context, token):
                self.build(context, token)
                return 0
        if self.match_Empty(context, token):
                self.build(context, token)
                return 0

        state_comment = "State: 0 - Start"
        token.detach
        expected_tokens = ["#EOF", "#Language", "#TagLine", "#FeatureLine", "#Comment", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 0

    # GherkinDocument:0>Feature:0>Feature_Header:0>#Language:0
    def match_token_at_1(self, token, context):
        if self.match_TagLine(context, token):
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 2
        if self.match_FeatureLine(context, token):
                self.build(context, token)
                return 3
        if self.match_Comment(context, token):
                self.build(context, token)
                return 1
        if self.match_Empty(context, token):
                self.build(context, token)
                return 1

        state_comment = "State: 1 - GherkinDocument:0>Feature:0>Feature_Header:0>#Language:0"
        token.detach
        expected_tokens = ["#TagLine", "#FeatureLine", "#Comment", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 1

    # GherkinDocument:0>Feature:0>Feature_Header:1>Tags:0>#TagLine:0
    def match_token_at_2(self, token, context):
        if self.match_TagLine(context, token):
                self.build(context, token)
                return 2
        if self.match_FeatureLine(context, token):
                self.end_rule(context, 'Tags')
                self.build(context, token)
                return 3
        if self.match_Comment(context, token):
                self.build(context, token)
                return 2
        if self.match_Empty(context, token):
                self.build(context, token)
                return 2

        state_comment = "State: 2 - GherkinDocument:0>Feature:0>Feature_Header:1>Tags:0>#TagLine:0"
        token.detach
        expected_tokens = ["#TagLine", "#FeatureLine", "#Comment", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 2

    # GherkinDocument:0>Feature:0>Feature_Header:2>#FeatureLine:0
    def match_token_at_3(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Feature_Header')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_Empty(context, token):
                self.build(context, token)
                return 3
        if self.match_Comment(context, token):
                self.build(context, token)
                return 5
        if self.match_BackgroundLine(context, token):
                self.end_rule(context, 'Feature_Header')
                self.start_rule(context, 'Background')
                self.build(context, token)
                return 6
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Feature_Header')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Feature_Header')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Feature_Header')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Other(context, token):
                self.start_rule(context, 'Description')
                self.build(context, token)
                return 4

        state_comment = "State: 3 - GherkinDocument:0>Feature:0>Feature_Header:2>#FeatureLine:0"
        token.detach
        expected_tokens = ["#EOF", "#Empty", "#Comment", "#BackgroundLine", "#TagLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Other"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 3

    # GherkinDocument:0>Feature:0>Feature_Header:3>Feature_Description:0>Description_Helper:1>Description:0>#Other:0
    def match_token_at_4(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Feature_Header')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_Comment(context, token):
                self.end_rule(context, 'Description')
                self.build(context, token)
                return 5
        if self.match_BackgroundLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Feature_Header')
                self.start_rule(context, 'Background')
                self.build(context, token)
                return 6
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Feature_Header')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Feature_Header')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Feature_Header')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Other(context, token):
                self.build(context, token)
                return 4

        state_comment = "State: 4 - GherkinDocument:0>Feature:0>Feature_Header:3>Feature_Description:0>Description_Helper:1>Description:0>#Other:0"
        token.detach
        expected_tokens = ["#EOF", "#Comment", "#BackgroundLine", "#TagLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Other"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 4

    # GherkinDocument:0>Feature:0>Feature_Header:3>Feature_Description:0>Description_Helper:2>#Comment:0
    def match_token_at_5(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Feature_Header')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_Comment(context, token):
                self.build(context, token)
                return 5
        if self.match_BackgroundLine(context, token):
                self.end_rule(context, 'Feature_Header')
                self.start_rule(context, 'Background')
                self.build(context, token)
                return 6
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Feature_Header')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Feature_Header')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Feature_Header')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Empty(context, token):
                self.build(context, token)
                return 5

        state_comment = "State: 5 - GherkinDocument:0>Feature:0>Feature_Header:3>Feature_Description:0>Description_Helper:2>#Comment:0"
        token.detach
        expected_tokens = ["#EOF", "#Comment", "#BackgroundLine", "#TagLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 5

    # GherkinDocument:0>Feature:1>Background:0>#BackgroundLine:0
    def match_token_at_6(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Background')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_Empty(context, token):
                self.build(context, token)
                return 6
        if self.match_Comment(context, token):
                self.build(context, token)
                return 8
        if self.match_StepLine(context, token):
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Other(context, token):
                self.start_rule(context, 'Description')
                self.build(context, token)
                return 7

        state_comment = "State: 6 - GherkinDocument:0>Feature:1>Background:0>#BackgroundLine:0"
        token.detach
        expected_tokens = ["#EOF", "#Empty", "#Comment", "#StepLine", "#TagLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Other"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 6

    # GherkinDocument:0>Feature:1>Background:1>Background_Description:0>Description_Helper:1>Description:0>#Other:0
    def match_token_at_7(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Background')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_Comment(context, token):
                self.end_rule(context, 'Description')
                self.build(context, token)
                return 8
        if self.match_StepLine(context, token):
                self.end_rule(context, 'Description')
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Other(context, token):
                self.build(context, token)
                return 7

        state_comment = "State: 7 - GherkinDocument:0>Feature:1>Background:1>Background_Description:0>Description_Helper:1>Description:0>#Other:0"
        token.detach
        expected_tokens = ["#EOF", "#Comment", "#StepLine", "#TagLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Other"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 7

    # GherkinDocument:0>Feature:1>Background:1>Background_Description:0>Description_Helper:2>#Comment:0
    def match_token_at_8(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Background')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_Comment(context, token):
                self.build(context, token)
                return 8
        if self.match_StepLine(context, token):
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Empty(context, token):
                self.build(context, token)
                return 8

        state_comment = "State: 8 - GherkinDocument:0>Feature:1>Background:1>Background_Description:0>Description_Helper:2>#Comment:0"
        token.detach
        expected_tokens = ["#EOF", "#Comment", "#StepLine", "#TagLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 8

    # GherkinDocument:0>Feature:1>Background:2>Scenario_Step:0>Step:0>#StepLine:0
    def match_token_at_9(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Background')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_TableRow(context, token):
                self.start_rule(context, 'DataTable')
                self.build(context, token)
                return 10
        if self.match_DocStringSeparator(context, token):
                self.start_rule(context, 'DocString')
                self.build(context, token)
                return 32
        if self.match_StepLine(context, token):
                self.end_rule(context, 'Step')
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Comment(context, token):
                self.build(context, token)
                return 9
        if self.match_Empty(context, token):
                self.build(context, token)
                return 9

        state_comment = "State: 9 - GherkinDocument:0>Feature:1>Background:2>Scenario_Step:0>Step:0>#StepLine:0"
        token.detach
        expected_tokens = ["#EOF", "#TableRow", "#DocStringSeparator", "#StepLine", "#TagLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Comment", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 9

    # GherkinDocument:0>Feature:1>Background:2>Scenario_Step:0>Step:1>Step_Arg:0>__alt1:0>DataTable:0>#TableRow:0
    def match_token_at_10(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Background')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_TableRow(context, token):
                self.build(context, token)
                return 10
        if self.match_StepLine(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Comment(context, token):
                self.build(context, token)
                return 10
        if self.match_Empty(context, token):
                self.build(context, token)
                return 10

        state_comment = "State: 10 - GherkinDocument:0>Feature:1>Background:2>Scenario_Step:0>Step:1>Step_Arg:0>__alt1:0>DataTable:0>#TableRow:0"
        token.detach
        expected_tokens = ["#EOF", "#TableRow", "#StepLine", "#TagLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Comment", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 10

    # GherkinDocument:0>Feature:2>Scenario_Definition:0>Tags:0>#TagLine:0
    def match_token_at_11(self, token, context):
        if self.match_TagLine(context, token):
                self.build(context, token)
                return 11
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Tags')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Tags')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Comment(context, token):
                self.build(context, token)
                return 11
        if self.match_Empty(context, token):
                self.build(context, token)
                return 11

        state_comment = "State: 11 - GherkinDocument:0>Feature:2>Scenario_Definition:0>Tags:0>#TagLine:0"
        token.detach
        expected_tokens = ["#TagLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Comment", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 11

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:0>Scenario:0>#ScenarioLine:0
    def match_token_at_12(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_Empty(context, token):
                self.build(context, token)
                return 12
        if self.match_Comment(context, token):
                self.build(context, token)
                return 14
        if self.match_StepLine(context, token):
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 15
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Other(context, token):
                self.start_rule(context, 'Description')
                self.build(context, token)
                return 13

        state_comment = "State: 12 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:0>Scenario:0>#ScenarioLine:0"
        token.detach
        expected_tokens = ["#EOF", "#Empty", "#Comment", "#StepLine", "#TagLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Other"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 12

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:0>Scenario:1>Scenario_Description:0>Description_Helper:1>Description:0>#Other:0
    def match_token_at_13(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_Comment(context, token):
                self.end_rule(context, 'Description')
                self.build(context, token)
                return 14
        if self.match_StepLine(context, token):
                self.end_rule(context, 'Description')
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 15
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Other(context, token):
                self.build(context, token)
                return 13

        state_comment = "State: 13 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:0>Scenario:1>Scenario_Description:0>Description_Helper:1>Description:0>#Other:0"
        token.detach
        expected_tokens = ["#EOF", "#Comment", "#StepLine", "#TagLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Other"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 13

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:0>Scenario:1>Scenario_Description:0>Description_Helper:2>#Comment:0
    def match_token_at_14(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_Comment(context, token):
                self.build(context, token)
                return 14
        if self.match_StepLine(context, token):
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 15
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Empty(context, token):
                self.build(context, token)
                return 14

        state_comment = "State: 14 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:0>Scenario:1>Scenario_Description:0>Description_Helper:2>#Comment:0"
        token.detach
        expected_tokens = ["#EOF", "#Comment", "#StepLine", "#TagLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 14

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:0>Scenario:2>Scenario_Step:0>Step:0>#StepLine:0
    def match_token_at_15(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_TableRow(context, token):
                self.start_rule(context, 'DataTable')
                self.build(context, token)
                return 16
        if self.match_DocStringSeparator(context, token):
                self.start_rule(context, 'DocString')
                self.build(context, token)
                return 30
        if self.match_StepLine(context, token):
                self.end_rule(context, 'Step')
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 15
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Comment(context, token):
                self.build(context, token)
                return 15
        if self.match_Empty(context, token):
                self.build(context, token)
                return 15

        state_comment = "State: 15 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:0>Scenario:2>Scenario_Step:0>Step:0>#StepLine:0"
        token.detach
        expected_tokens = ["#EOF", "#TableRow", "#DocStringSeparator", "#StepLine", "#TagLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Comment", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 15

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:0>Scenario:2>Scenario_Step:0>Step:1>Step_Arg:0>__alt1:0>DataTable:0>#TableRow:0
    def match_token_at_16(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_TableRow(context, token):
                self.build(context, token)
                return 16
        if self.match_StepLine(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 15
        if self.match_TagLine(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Comment(context, token):
                self.build(context, token)
                return 16
        if self.match_Empty(context, token):
                self.build(context, token)
                return 16

        state_comment = "State: 16 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:0>Scenario:2>Scenario_Step:0>Step:1>Step_Arg:0>__alt1:0>DataTable:0>#TableRow:0"
        token.detach
        expected_tokens = ["#EOF", "#TableRow", "#StepLine", "#TagLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Comment", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 16

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:0>#ScenarioOutlineLine:0
    def match_token_at_17(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_Empty(context, token):
                self.build(context, token)
                return 17
        if self.match_Comment(context, token):
                self.build(context, token)
                return 19
        if self.match_StepLine(context, token):
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 20
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 22
        if self.match_TagLine(context, token):
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ExamplesLine(context, token):
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples')
                self.build(context, token)
                return 23
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Other(context, token):
                self.start_rule(context, 'Description')
                self.build(context, token)
                return 18

        state_comment = "State: 17 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:0>#ScenarioOutlineLine:0"
        token.detach
        expected_tokens = ["#EOF", "#Empty", "#Comment", "#StepLine", "#TagLine", "#ExamplesLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Other"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 17

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:1>ScenarioOutline_Description:0>Description_Helper:1>Description:0>#Other:0
    def match_token_at_18(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_Comment(context, token):
                self.end_rule(context, 'Description')
                self.build(context, token)
                return 19
        if self.match_StepLine(context, token):
                self.end_rule(context, 'Description')
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 20
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, 'Description')
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 22
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ExamplesLine(context, token):
                self.end_rule(context, 'Description')
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples')
                self.build(context, token)
                return 23
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Other(context, token):
                self.build(context, token)
                return 18

        state_comment = "State: 18 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:1>ScenarioOutline_Description:0>Description_Helper:1>Description:0>#Other:0"
        token.detach
        expected_tokens = ["#EOF", "#Comment", "#StepLine", "#TagLine", "#ExamplesLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Other"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 18

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:1>ScenarioOutline_Description:0>Description_Helper:2>#Comment:0
    def match_token_at_19(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_Comment(context, token):
                self.build(context, token)
                return 19
        if self.match_StepLine(context, token):
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 20
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 22
        if self.match_TagLine(context, token):
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ExamplesLine(context, token):
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples')
                self.build(context, token)
                return 23
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Empty(context, token):
                self.build(context, token)
                return 19

        state_comment = "State: 19 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:1>ScenarioOutline_Description:0>Description_Helper:2>#Comment:0"
        token.detach
        expected_tokens = ["#EOF", "#Comment", "#StepLine", "#TagLine", "#ExamplesLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 19

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:2>ScenarioOutline_Step:0>Step:0>#StepLine:0
    def match_token_at_20(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Step')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_TableRow(context, token):
                self.start_rule(context, 'DataTable')
                self.build(context, token)
                return 21
        if self.match_DocStringSeparator(context, token):
                self.start_rule(context, 'DocString')
                self.build(context, token)
                return 28
        if self.match_StepLine(context, token):
                self.end_rule(context, 'Step')
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 20
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, 'Step')
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 22
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Step')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ExamplesLine(context, token):
                self.end_rule(context, 'Step')
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples')
                self.build(context, token)
                return 23
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Step')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Step')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Comment(context, token):
                self.build(context, token)
                return 20
        if self.match_Empty(context, token):
                self.build(context, token)
                return 20

        state_comment = "State: 20 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:2>ScenarioOutline_Step:0>Step:0>#StepLine:0"
        token.detach
        expected_tokens = ["#EOF", "#TableRow", "#DocStringSeparator", "#StepLine", "#TagLine", "#ExamplesLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Comment", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 20

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:2>ScenarioOutline_Step:0>Step:1>Step_Arg:0>__alt1:0>DataTable:0>#TableRow:0
    def match_token_at_21(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_TableRow(context, token):
                self.build(context, token)
                return 21
        if self.match_StepLine(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 20
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 22
        if self.match_TagLine(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ExamplesLine(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples')
                self.build(context, token)
                return 23
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'DataTable')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Comment(context, token):
                self.build(context, token)
                return 21
        if self.match_Empty(context, token):
                self.build(context, token)
                return 21

        state_comment = "State: 21 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:2>ScenarioOutline_Step:0>Step:1>Step_Arg:0>__alt1:0>DataTable:0>#TableRow:0"
        token.detach
        expected_tokens = ["#EOF", "#TableRow", "#StepLine", "#TagLine", "#ExamplesLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Comment", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 21

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:3>Examples_Definition:0>Tags:0>#TagLine:0
    def match_token_at_22(self, token, context):
        if self.match_TagLine(context, token):
                self.build(context, token)
                return 22
        if self.match_ExamplesLine(context, token):
                self.end_rule(context, 'Tags')
                self.start_rule(context, 'Examples')
                self.build(context, token)
                return 23
        if self.match_Comment(context, token):
                self.build(context, token)
                return 22
        if self.match_Empty(context, token):
                self.build(context, token)
                return 22

        state_comment = "State: 22 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:3>Examples_Definition:0>Tags:0>#TagLine:0"
        token.detach
        expected_tokens = ["#TagLine", "#ExamplesLine", "#Comment", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 22

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:3>Examples_Definition:1>Examples:0>#ExamplesLine:0
    def match_token_at_23(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_Empty(context, token):
                self.build(context, token)
                return 23
        if self.match_Comment(context, token):
                self.build(context, token)
                return 25
        if self.match_TableRow(context, token):
                self.start_rule(context, 'Examples_Table')
                self.build(context, token)
                return 26
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 22
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ExamplesLine(context, token):
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples')
                self.build(context, token)
                return 23
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Other(context, token):
                self.start_rule(context, 'Description')
                self.build(context, token)
                return 24

        state_comment = "State: 23 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:3>Examples_Definition:1>Examples:0>#ExamplesLine:0"
        token.detach
        expected_tokens = ["#EOF", "#Empty", "#Comment", "#TableRow", "#TagLine", "#ExamplesLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Other"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 23

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:3>Examples_Definition:1>Examples:1>Examples_Description:0>Description_Helper:1>Description:0>#Other:0
    def match_token_at_24(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_Comment(context, token):
                self.end_rule(context, 'Description')
                self.build(context, token)
                return 25
        if self.match_TableRow(context, token):
                self.end_rule(context, 'Description')
                self.start_rule(context, 'Examples_Table')
                self.build(context, token)
                return 26
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 22
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ExamplesLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples')
                self.build(context, token)
                return 23
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Description')
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Other(context, token):
                self.build(context, token)
                return 24

        state_comment = "State: 24 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:3>Examples_Definition:1>Examples:1>Examples_Description:0>Description_Helper:1>Description:0>#Other:0"
        token.detach
        expected_tokens = ["#EOF", "#Comment", "#TableRow", "#TagLine", "#ExamplesLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Other"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 24

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:3>Examples_Definition:1>Examples:1>Examples_Description:0>Description_Helper:2>#Comment:0
    def match_token_at_25(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_Comment(context, token):
                self.build(context, token)
                return 25
        if self.match_TableRow(context, token):
                self.start_rule(context, 'Examples_Table')
                self.build(context, token)
                return 26
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 22
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ExamplesLine(context, token):
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples')
                self.build(context, token)
                return 23
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Empty(context, token):
                self.build(context, token)
                return 25

        state_comment = "State: 25 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:3>Examples_Definition:1>Examples:1>Examples_Description:0>Description_Helper:2>#Comment:0"
        token.detach
        expected_tokens = ["#EOF", "#Comment", "#TableRow", "#TagLine", "#ExamplesLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 25

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:3>Examples_Definition:1>Examples:2>Examples_Table:0>#TableRow:0
    def match_token_at_26(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'Examples_Table')
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_TableRow(context, token):
                self.build(context, token)
                return 26
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, 'Examples_Table')
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 22
        if self.match_TagLine(context, token):
                self.end_rule(context, 'Examples_Table')
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ExamplesLine(context, token):
                self.end_rule(context, 'Examples_Table')
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples')
                self.build(context, token)
                return 23
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'Examples_Table')
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'Examples_Table')
                self.end_rule(context, 'Examples')
                self.end_rule(context, 'Examples_Definition')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Comment(context, token):
                self.build(context, token)
                return 26
        if self.match_Empty(context, token):
                self.build(context, token)
                return 26

        state_comment = "State: 26 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:3>Examples_Definition:1>Examples:2>Examples_Table:0>#TableRow:0"
        token.detach
        expected_tokens = ["#EOF", "#TableRow", "#TagLine", "#ExamplesLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Comment", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 26

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:2>ScenarioOutline_Step:0>Step:1>Step_Arg:0>__alt1:1>DocString:0>#DocStringSeparator:0
    def match_token_at_28(self, token, context):
        if self.match_DocStringSeparator(context, token):
                self.build(context, token)
                return 29
        if self.match_Other(context, token):
                self.build(context, token)
                return 28

        state_comment = "State: 28 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:2>ScenarioOutline_Step:0>Step:1>Step_Arg:0>__alt1:1>DocString:0>#DocStringSeparator:0"
        token.detach
        expected_tokens = ["#DocStringSeparator", "#Other"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 28

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:2>ScenarioOutline_Step:0>Step:1>Step_Arg:0>__alt1:1>DocString:2>#DocStringSeparator:0
    def match_token_at_29(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_StepLine(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 20
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 22
        if self.match_TagLine(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ExamplesLine(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.start_rule(context, 'Examples_Definition')
                self.start_rule(context, 'Examples')
                self.build(context, token)
                return 23
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'ScenarioOutline')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Comment(context, token):
                self.build(context, token)
                return 29
        if self.match_Empty(context, token):
                self.build(context, token)
                return 29

        state_comment = "State: 29 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:1>ScenarioOutline:2>ScenarioOutline_Step:0>Step:1>Step_Arg:0>__alt1:1>DocString:2>#DocStringSeparator:0"
        token.detach
        expected_tokens = ["#EOF", "#StepLine", "#TagLine", "#ExamplesLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Comment", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 29

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:0>Scenario:2>Scenario_Step:0>Step:1>Step_Arg:0>__alt1:1>DocString:0>#DocStringSeparator:0
    def match_token_at_30(self, token, context):
        if self.match_DocStringSeparator(context, token):
                self.build(context, token)
                return 31
        if self.match_Other(context, token):
                self.build(context, token)
                return 30

        state_comment = "State: 30 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:0>Scenario:2>Scenario_Step:0>Step:1>Step_Arg:0>__alt1:1>DocString:0>#DocStringSeparator:0"
        token.detach
        expected_tokens = ["#DocStringSeparator", "#Other"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 30

    # GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:0>Scenario:2>Scenario_Step:0>Step:1>Step_Arg:0>__alt1:1>DocString:2>#DocStringSeparator:0
    def match_token_at_31(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_StepLine(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 15
        if self.match_TagLine(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Scenario')
                self.end_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Comment(context, token):
                self.build(context, token)
                return 31
        if self.match_Empty(context, token):
                self.build(context, token)
                return 31

        state_comment = "State: 31 - GherkinDocument:0>Feature:2>Scenario_Definition:1>__alt0:0>Scenario:2>Scenario_Step:0>Step:1>Step_Arg:0>__alt1:1>DocString:2>#DocStringSeparator:0"
        token.detach
        expected_tokens = ["#EOF", "#StepLine", "#TagLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Comment", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 31

    # GherkinDocument:0>Feature:1>Background:2>Scenario_Step:0>Step:1>Step_Arg:0>__alt1:1>DocString:0>#DocStringSeparator:0
    def match_token_at_32(self, token, context):
        if self.match_DocStringSeparator(context, token):
                self.build(context, token)
                return 33
        if self.match_Other(context, token):
                self.build(context, token)
                return 32

        state_comment = "State: 32 - GherkinDocument:0>Feature:1>Background:2>Scenario_Step:0>Step:1>Step_Arg:0>__alt1:1>DocString:0>#DocStringSeparator:0"
        token.detach
        expected_tokens = ["#DocStringSeparator", "#Other"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 32

    # GherkinDocument:0>Feature:1>Background:2>Scenario_Step:0>Step:1>Step_Arg:0>__alt1:1>DocString:2>#DocStringSeparator:0
    def match_token_at_33(self, token, context):
        if self.match_EOF(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Background')
                self.end_rule(context, 'Feature')
                self.build(context, token)
                return 27
        if self.match_StepLine(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.start_rule(context, 'Step')
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Tags')
                self.build(context, token)
                return 11
        if self.match_ScenarioLine(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'Scenario')
                self.build(context, token)
                return 12
        if self.match_ScenarioOutlineLine(context, token):
                self.end_rule(context, 'DocString')
                self.end_rule(context, 'Step')
                self.end_rule(context, 'Background')
                self.start_rule(context, 'Scenario_Definition')
                self.start_rule(context, 'ScenarioOutline')
                self.build(context, token)
                return 17
        if self.match_Comment(context, token):
                self.build(context, token)
                return 33
        if self.match_Empty(context, token):
                self.build(context, token)
                return 33

        state_comment = "State: 33 - GherkinDocument:0>Feature:1>Background:2>Scenario_Step:0>Step:1>Step_Arg:0>__alt1:1>DocString:2>#DocStringSeparator:0"
        token.detach
        expected_tokens = ["#EOF", "#StepLine", "#TagLine", "#ScenarioLine", "#ScenarioOutlineLine", "#Comment", "#Empty"]
        error = UnexpectedEOFException(token, expected_tokens, state_comment) if token.eof() else UnexpectedTokenException(token, expected_tokens, state_comment)
        if (self.stop_at_first_error):
            raise error
        self.add_error(context, error)
        return 33

    def lookahead_0(self, context, currentToken):
        currentToken.detach
        token = None
        queue = []
        match = False
        while True:
            token = self.read_token(context)
            token.detach
            queue.append(token)

            if (self.match_ExamplesLine(context, token) or False):
                match = True
                break

            if not (self.match_Empty(context, token) or self.match_Comment(context, token) or self.match_TagLine(context, token) or False):
                break

        context.token_queue.extend(queue)

        return match

    # private

    def handle_ast_error(self, context, argument, action):
        self.handle_external_error(context, True, argument, action)

    def handle_external_error(self, context, default_value, argument, action):
        if self.stop_at_first_error:
            return action(argument)

        try:
            return action(argument)
        except CompositeParserException as e:
            for error in e.errors:
                self.add_error(context, error)
        except ParserException as e:
            self.add_error(context, e)
        return default_value



from __future__ import print_function
import json
from .parser import Parser
from .token_scanner import TokenScanner
from .pickles.compiler import compile
from .errors import ParserException, CompositeParserException

class Inout(object):
    def __init__(self, print_source, print_ast, print_pickles):
        self.print_source = print_source
        self.print_ast = print_ast
        self.print_pickles = print_pickles
        self.parser = Parser()
        self.parser.stop_at_first_error = False

    def process(self, input, output):
        line = input.readline().rstrip()
        event = json.loads(line)
        if (event['type'] == 'source'):
            uri = event['uri']
            source = event['data']
            token_scanner = TokenScanner(source)

            try:
                gherkin_document = self.parser.parse(token_scanner)
                if (self.print_source):
                    print(line, file=output)
                if (self.print_ast):
                    print(json.dumps(gherkin_document), file=output)
                if (self.print_pickles):
                    pickles = compile(gherkin_document, uri)
                    for pickle in pickles:
                        print(json.dumps(pickle), file=output)
            except CompositeParserException as e:
                self.print_errors(output, e.errors, uri)
            except ParserException as e:
                self.print_errors(output, [e], uri)

    def print_errors(self, output, errors, uri):
        for error in errors:
            attachment = {
                'type': "attachment",
                'source': {
                    'uri': uri,
                    'start': {
                        'line': error.location['line'],
                        'column': error.location['column']
                    }
                },
                'data': error.message,
                'media': {
                    'encoding': "utf-8",
                    'type': "text/vnd.cucumber.stacktrace+plain"
                }
            }
            print(json.dumps(attachment), file=output)



import re
from .dialect import Dialect
from .errors import NoSuchLanguageException


class TokenMatcher(object):
    LANGUAGE_RE = re.compile(r"^\s*#\s*language\s*:\s*([a-zA-Z\-_]+)\s*$")

    def __init__(self, dialect_name='en'):
        self._default_dialect_name = dialect_name
        self._change_dialect(dialect_name)
        self.reset()

    def reset(self):
        if self.dialect_name != self._default_dialect_name:
            self._change_dialect(self._default_dialect_name)
        self._indent_to_remove = 0
        self._active_doc_string_separator = None

    def match_FeatureLine(self, token):
        return self._match_title_line(token, 'FeatureLine', self.dialect.feature_keywords)

    def match_ScenarioLine(self, token):
        return self._match_title_line(token, 'ScenarioLine', self.dialect.scenario_keywords)

    def match_ScenarioOutlineLine(self, token):
        return self._match_title_line(token, 'ScenarioOutlineLine',
                                      self.dialect.scenario_outline_keywords)

    def match_BackgroundLine(self, token):
        return self._match_title_line(token, 'BackgroundLine', self.dialect.background_keywords)

    def match_ExamplesLine(self, token):
        return self._match_title_line(token, 'ExamplesLine', self.dialect.examples_keywords)

    def match_TableRow(self, token):
        if not token.line.startswith('|'):
            return False
        # TODO: indent
        self._set_token_matched(token, 'TableRow', items=token.line.table_cells)
        return True

    def match_StepLine(self, token):
        keywords = (self.dialect.given_keywords +
                    self.dialect.when_keywords +
                    self.dialect.then_keywords +
                    self.dialect.and_keywords +
                    self.dialect.but_keywords)
        for keyword in (k for k in keywords if token.line.startswith(k)):
            title = token.line.get_rest_trimmed(len(keyword))
            self._set_token_matched(token, 'StepLine', title, keyword)
            return True

        return False

    def match_Comment(self, token):
        if not token.line.startswith('#'):
            return False

        text = token.line._line_text  # take the entire line, including leading space
        self._set_token_matched(token, 'Comment', text, indent=0)
        return True

    def match_Empty(self, token):
        if not token.line.is_empty():
            return False

        self._set_token_matched(token, 'Empty', indent=0)
        return True

    def match_Language(self, token):
        match = self.LANGUAGE_RE.match(token.line.get_line_text())
        if not match:
            return False

        dialect_name = match.group(1)
        self._set_token_matched(token, 'Language', dialect_name)
        self._change_dialect(dialect_name, token.location)
        return True

    def match_TagLine(self, token):
        if not token.line.startswith('@'):
            return False

        self._set_token_matched(token, 'TagLine', items=token.line.tags)
        return True

    def match_DocStringSeparator(self, token):
        if not self._active_doc_string_separator:
            # open
            return (self._match_DocStringSeparator(token, '"""', True) or
                    self._match_DocStringSeparator(token, '```', True))
        else:
            # close
            return self._match_DocStringSeparator(token, self._active_doc_string_separator, False)

    def _match_DocStringSeparator(self, token, separator, is_open):
        if not token.line.startswith(separator):
            return False

        content_type = None
        if is_open:
            content_type = token.line.get_rest_trimmed(len(separator))
            self._active_doc_string_separator = separator
            self._indent_to_remove = token.line.indent
        else:
            self._active_doc_string_separator = None
            self._indent_to_remove = 0

        # TODO: Use the separator as keyword. That's needed for pretty printing.
        self._set_token_matched(token, 'DocStringSeparator', content_type)
        return True

    def match_Other(self, token):
        # take the entire line, except removing DocString indents
        text = token.line.get_line_text(self._indent_to_remove)
        self._set_token_matched(token, 'Other', self._unescaped_docstring(text), indent=0)
        return True

    def match_EOF(self, token):
        if not token.eof():
            return False

        self._set_token_matched(token, 'EOF')
        return True

    def _match_title_line(self, token, token_type, keywords):
        for keyword in (k for k in keywords if token.line.startswith_title_keyword(k)):
            title = token.line.get_rest_trimmed(len(keyword) + len(':'))
            self._set_token_matched(token, token_type, title, keyword)
            return True

        return False

    def _set_token_matched(self, token, matched_type, text=None,
                           keyword=None, indent=None, items=None):
        if items is None:
            items = []
        token.matched_type = matched_type
        # text == '' should not result in None
        token.matched_text = text.rstrip('\r\n') if text is not None else None
        token.matched_keyword = keyword
        if indent is not None:
            token.matched_indent = indent
        else:
            token.matched_indent = token.line.indent if token.line else 0
        token.matched_items = items
        token.location['column'] = token.matched_indent + 1
        token.matched_gherkin_dialect = self.dialect_name

    def _change_dialect(self, dialect_name, location=None):
        dialect = Dialect.for_name(dialect_name)
        if not dialect:
            raise NoSuchLanguageException(dialect_name, location)

        self.dialect_name = dialect_name
        self.dialect = dialect

    def _unescaped_docstring(self, text):
        return text.replace('\\"\\"\\"', '"""') if self._active_doc_string_separator else text



import io
import os
import json


DIALECT_FILE_PATH = os.path.join(
    os.path.dirname(__file__),
    'gherkin-languages.json')

with io.open(DIALECT_FILE_PATH, 'r', encoding='utf-8') as file:
    DIALECTS = json.load(file)


class Dialect(object):

    @classmethod
    def for_name(cls, name):
        return cls(DIALECTS[name]) if name in DIALECTS else None

    def __init__(self, spec):
        self.spec = spec

    @property
    def feature_keywords(self):
        return self.spec['feature']

    @property
    def scenario_keywords(self):
        return self.spec['scenario']

    @property
    def scenario_outline_keywords(self):
        return self.spec['scenarioOutline']

    @property
    def background_keywords(self):
        return self.spec['background']

    @property
    def examples_keywords(self):
        return self.spec['examples']

    @property
    def given_keywords(self):
        return self.spec['given']

    @property
    def when_keywords(self):
        return self.spec['when']

    @property
    def then_keywords(self):
        return self.spec['then']

    @property
    def and_keywords(self):
        return self.spec['and']

    @property
    def but_keywords(self):
        return self.spec['but']



import io

def source_event(path):
    event = {
        'type': 'source',
        'uri': path,
        'data': io.open(path, 'rU', encoding='utf8', newline='').read(),
        'media': {
            'encoding': 'utf-8',
            'type': 'text/vnd.cucumber.gherkin+plain'
        }
    }
    return event

class SourceEvents:
    def __init__(self, paths):
        self.paths = paths

    def enum(self):
        return map(source_event, self.paths)



from .ast_node import AstNode
from .errors import AstBuilderException


class AstBuilder(object):
    def __init__(self):
        self.reset()

    def reset(self):
        self.stack = [AstNode('None')]
        self.comments = []

    def start_rule(self, rule_type):
        self.stack.append(AstNode(rule_type))

    def end_rule(self, rule_type):
        node = self.stack.pop()
        self.current_node.add(node.rule_type, self.transform_node(node))

    def build(self, token):
        if token.matched_type == 'Comment':
            self.comments.append({
                'type': 'Comment',
                'location': self.get_location(token),
                'text': token.matched_text
            })
        else:
            self.current_node.add(token.matched_type, token)

    def get_result(self):
        return self.current_node.get_single('GherkinDocument')

    @property
    def current_node(self):
        return self.stack[-1]

    def get_location(self, token, column=None):
        return (token.location if not column else
                {'line': token.location['line'], 'column': column})

    def get_tags(self, node):
        tags = []
        tags_node = node.get_single('Tags')
        if not tags_node:
            return tags

        for token in tags_node.get_tokens('TagLine'):
            tags += [{'type': 'Tag',
                      'location': self.get_location(token, tag_item['column']),
                      'name': tag_item['text']} for tag_item in token.matched_items]

        return tags

    def get_table_rows(self, node):
        rows = [{'type': 'TableRow',
                 'location': self.get_location(token),
                 'cells': self.get_cells(token)} for token in node.get_tokens('TableRow')]
        self.ensure_cell_count(rows)
        return rows

    def ensure_cell_count(self, rows):
        if not rows:
            return

        cell_count = len(rows[0]['cells'])
        for row in rows:
            if len(row['cells']) != cell_count:
                raise AstBuilderException("inconsistent cell count within the table",
                                          row['location'])

    def get_cells(self, table_row_token):
        return [{'type': 'TableCell',
                 'location': self.get_location(table_row_token, cell_item['column']),
                 'value': cell_item['text']} for cell_item in table_row_token.matched_items]

    def get_description(self, node):
        return node.get_single('Description')

    def get_steps(self, node):
        return node.get_items('Step')

    def transform_node(self, node):
        if node.rule_type == 'Step':
            step_line = node.get_token('StepLine')
            step_argument = None
            if node.get_single('DataTable'):
                step_argument = node.get_single('DataTable')
            elif node.get_single('DocString'):
                step_argument = node.get_single('DocString')

            return self.reject_nones({
                'type': node.rule_type,
                'location': self.get_location(step_line),
                'keyword': step_line.matched_keyword,
                'text': step_line.matched_text,
                'argument': step_argument
            })
        elif node.rule_type == 'DocString':
            separator_token = node.get_tokens('DocStringSeparator')[0]
            content_type = (separator_token.matched_text if len(separator_token.matched_text) > 0
                            else None)
            line_tokens = node.get_tokens('Other')
            content = '\n'.join([t.matched_text for t in line_tokens])

            return self.reject_nones({
                'type': node.rule_type,
                'location': self.get_location(separator_token),
                'contentType': content_type,
                'content': content
            })
        elif node.rule_type == 'DataTable':
            rows = self.get_table_rows(node)
            return self.reject_nones({
                'type': node.rule_type,
                'location': rows[0]['location'],
                'rows': rows,
            })
        elif node.rule_type == 'Background':
            background_line = node.get_token('BackgroundLine')
            description = self.get_description(node)
            steps = self.get_steps(node)

            return self.reject_nones({
                'type': node.rule_type,
                'location': self.get_location(background_line),
                'keyword': background_line.matched_keyword,
                'name': background_line.matched_text,
                'description': description,
                'steps': steps
            })
        elif node.rule_type == 'Scenario_Definition':
            tags = self.get_tags(node)
            scenario_node = node.get_single('Scenario')
            if scenario_node:
                scenario_line = scenario_node.get_token('ScenarioLine')
                description = self.get_description(scenario_node)
                steps = self.get_steps(scenario_node)

                return self.reject_nones({
                    'type': scenario_node.rule_type,
                    'tags': tags,
                    'location': self.get_location(scenario_line),
                    'keyword': scenario_line.matched_keyword,
                    'name': scenario_line.matched_text,
                    'description': description,
                    'steps': steps
                })
            else:
                scenario_outline_node = node.get_single('ScenarioOutline')
                if not scenario_outline_node:
                    raise RuntimeError('Internal grammar error')

                scenario_outline_line = scenario_outline_node.get_token('ScenarioOutlineLine')
                description = self.get_description(scenario_outline_node)
                steps = self.get_steps(scenario_outline_node)
                examples = scenario_outline_node.get_items('Examples_Definition')

                return self.reject_nones({
                    'type': scenario_outline_node.rule_type,
                    'tags': tags,
                    'location': self.get_location(scenario_outline_line),
                    'keyword': scenario_outline_line.matched_keyword,
                    'name': scenario_outline_line.matched_text,
                    'description': description,
                    'steps': steps,
                    'examples': examples
                })
        elif node.rule_type == 'Examples_Definition':
            tags = self.get_tags(node)
            examples_node = node.get_single('Examples')
            examples_line = examples_node.get_token('ExamplesLine')
            description = self.get_description(examples_node)
            examples_table = examples_node.get_single('Examples_Table')

            return self.reject_nones({
                'type': examples_node.rule_type,
                'tags': tags,
                'location': self.get_location(examples_line),
                'keyword': examples_line.matched_keyword,
                'name': examples_line.matched_text,
                'description': description,
                'tableHeader': examples_table['tableHeader'] if examples_table else None,
                'tableBody': examples_table['tableBody'] if examples_table else None
            })
        elif node.rule_type == 'Examples_Table':
            rows = self.get_table_rows(node)
            return self.reject_nones({
                'tableHeader': rows[0],
                'tableBody': rows[1:]
            })
        elif node.rule_type == 'Description':
            line_tokens = node.get_tokens('Other')
            # Trim trailing empty lines
            last_non_empty = next(i for i, j in reversed(list(enumerate(line_tokens)))
                                  if j.matched_text)
            description = '\n'.join([token.matched_text for token in
                                     line_tokens[:last_non_empty + 1]])

            return description
        elif node.rule_type == 'Feature':
            header = node.get_single('Feature_Header')
            if not header:
                return

            tags = self.get_tags(header)
            feature_line = header.get_token('FeatureLine')
            if not feature_line:
                return

            children = []
            background = node.get_single('Background')
            if (background):
                children.append(background)
            children = children + node.get_items('Scenario_Definition')
            description = self.get_description(header)
            language = feature_line.matched_gherkin_dialect

            return self.reject_nones({
                'type': node.rule_type,
                'tags': tags,
                'location': self.get_location(feature_line),
                'language': language,
                'keyword': feature_line.matched_keyword,
                'name': feature_line.matched_text,
                'description': description,
                'children': children
            })
        elif node.rule_type == 'GherkinDocument':
            feature = node.get_single('Feature')

            return self.reject_nones({
                'type': node.rule_type,
                'feature': feature,
                'comments': self.comments
            })
        else:
            return node

    def reject_nones(self, values):
        return {k: v for k, v in values.items() if v is not None}  # only None should be rejected



import os
from optparse import OptionParser
import sys
if sys.version_info < (3, 0) and os.name != 'nt':
    import codecs
    UTF8Writer = codecs.getwriter('utf8')
    sys.stdout = UTF8Writer(sys.stdout)

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
import json
from gherkin.stream.gherkin_events import GherkinEvents
from gherkin.stream.source_events import SourceEvents

parser = OptionParser()
parser.add_option("--no-source",  action="store_false", dest="print_source",  default=True, help="don't print source events")
parser.add_option("--no-ast",     action="store_false", dest="print_ast",     default=True, help="don't print ast events")
parser.add_option("--no-pickles", action="store_false", dest="print_pickles", default=True, help="don't print pickle events")

(options, args) = parser.parse_args()

source_events = SourceEvents(args)
gherkin_events = GherkinEvents(options)

for source_event in source_events.enum():
    for event in gherkin_events.enum(source_event):
        print(json.dumps(event))



class ParserError(Exception):
    pass


class ParserException(ParserError):
    def __init__(self, message, location):
        self.location = location
        super(ParserException, self).__init__('(' + str(location['line']) + ':' +
                                              str(location['column'] if 'column' in
                                                  location else 0) + '): ' + message)


class NoSuchLanguageException(ParserException):
    def __init__(self, language, location):
        super(NoSuchLanguageException, self).__init__('Language not supported: ' + language,
                                                      location)


class AstBuilderException(ParserException):
    pass


class UnexpectedEOFException(ParserException):
    def __init__(self, received_token, expected_token_types, state_comment):
        message = 'unexpected end of file, expected: ' + ', '.join(expected_token_types)
        super(UnexpectedEOFException, self).__init__(message, received_token.location)


class UnexpectedTokenException(ParserException):
    def __init__(self, received_token, expected_token_types, state_comment):
        message = ("expected: " + ', '.join(expected_token_types) + ", got '" +
                   received_token.token_value().strip() + "'")
        column = received_token.location['column'] if 'column' in received_token.location else None
        location = (received_token.location if column else
                    {'line': received_token.location['line'],
                     'column': received_token.line.indent + 1})
        super(UnexpectedTokenException, self).__init__(message, location)


class CompositeParserException(ParserError):
    def __init__(self, errors):
        self.errors = errors
        super(CompositeParserException, self).__init__("Parser errors:\n" +
                                                       '\n'.join([error.args[0] for error in
                                                                  errors]))



class GherkinLine(object):
    def __init__(self, line_text, line_number):
        self._line_text = line_text
        self._line_number = line_number
        self._trimmed_line_text = line_text.lstrip()
        self.indent = len(line_text) - len(self._trimmed_line_text)

    def get_rest_trimmed(self, length):
        return self._trimmed_line_text[length:].strip()

    def get_line_text(self, indent_to_remove=-1):
        if indent_to_remove < 0 or indent_to_remove > self.indent:
            return self._trimmed_line_text
        else:
            return self._line_text[indent_to_remove:]

    def is_empty(self):
        return not self._trimmed_line_text

    def startswith(self, prefix):
        return self._trimmed_line_text.startswith(prefix)

    def startswith_title_keyword(self, keyword):
        return self._trimmed_line_text.startswith(keyword + ':')

    @property
    def table_cells(self):
        cells = []
        for cell, col in self.split_table_cells(self._trimmed_line_text.strip()):
            cell_indent = len(cell) - len(cell.lstrip())
            cells.append({'column': col + self.indent + cell_indent, 'text': cell.strip()})
        return cells

    def split_table_cells(self, row):
        """
        An iterator returning all the table cells in a row with their positions,
        accounting for escaping.
        """

        row = iter(row)
        col = 0
        start_col = col + 1
        cell = ''
        first_cell = True
        while True:
            char = next(row, None)
            col += 1
            if char == '|':
                if first_cell:
                    # First cell (content before the first |) is skipped
                    first_cell = False
                else:
                    yield (cell, start_col)
                cell = ''
                start_col = col + 1
            elif char == '\\':
                char = next(row)
                col += 1
                if char == 'n':
                    cell += '\n'
                else:
                    if char not in ['|', '\\']:
                        cell += '\\'
                    cell += char
            elif char:
                cell += char
            else:
                break
        # Last cell (content after the last |) is skipped

    @property
    def tags(self):
        column = self.indent + 1
        items = self._trimmed_line_text.strip().split('@')
        tags = []
        for item in items[1:]:
            tags.append({'column': column, 'text': '@' + item.strip()})
            column += len(item) + 1
        return tags



from collections import defaultdict


class AstNode(object):

    def __init__(self, rule_type):
        self.rule_type = rule_type
        self._sub_items = defaultdict(list)

    def add(self, rule_type, obj):
        self._sub_items[rule_type].append(obj)

    def get_single(self, rule_type):
        return self._sub_items[rule_type][0] if self._sub_items[rule_type] else None

    def get_items(self, rule_type):
        return self._sub_items[rule_type]

    def get_token(self, token_type):
        return self.get_single(token_type)

    def get_tokens(self, token_type):
        return self._sub_items[token_type]



import io
import os
import sys
from .token import Token
from .gherkin_line import GherkinLine


class TokenScanner(object):
    """
    The scanner reads a gherkin doc (typically read from a `.feature` file) and creates a token for
    each line.

    The tokens are passed to the parser, which outputs an AST (Abstract Syntax Tree).

    If the scanner sees a `#` language header, it will reconfigure itself dynamically to look for
    Gherkin keywords for the associated language. The keywords are defined in
    :file:`gherkin-languages.json`.
    """

    def __init__(self, path_or_str):
        if os.path.exists(path_or_str):
            self.io = io.open(path_or_str, 'rU', encoding='utf8')
        else:
            if sys.version_info < (3, 0):
                if isinstance(path_or_str, str):
                    self.io = io.StringIO(unicode(path_or_str, encoding='utf8'))
                else:
                    self.io = io.StringIO(path_or_str)
            else:
                self.io = io.StringIO(path_or_str)
        self.line_number = 0

    def read(self):
        self.line_number += 1
        location = {'line': self.line_number, 'column': 0}
        line = self.io.readline()
        return Token((GherkinLine(line, self.line_number) if line else line), location)

    def __del__(self):
        # close file descriptor if it's still open
        try:
            self.io.close()
        except AttributeError:
            pass



class Token(object):
    def __init__(self, gherkin_line, location):
        self.line = gherkin_line
        self.location = location

    def eof(self):
        return not self.line

    def detach(self):
        pass  # TODO: detach line - is this needed?

    def token_value(self):
        return "EOF" if self.eof() else self.line.get_line_text()



import re

from ..count_symbols import count_symbols


def compile(gherkin_document):
    pickles = []
    if 'feature' not in gherkin_document:
        return pickles

    feature = gherkin_document['feature']
    feature_tags = feature['tags']
    language = feature['language']
    background_steps = []
    for scenario_definition in feature['children']:
        args = (feature_tags, background_steps, scenario_definition, language, pickles)
        if scenario_definition['type'] is 'Background':
            background_steps = _pickle_steps(scenario_definition)
        elif scenario_definition['type'] is 'Scenario':
            _compile_scenario(*args)
        else:
            _compile_scenario_outline(*args)
    return pickles


def _compile_scenario(feature_tags, background_steps, scenario, language, pickles):
    if len(scenario['steps']) == 0:
        return

    steps = list(background_steps)
    tags = list(feature_tags) + list(scenario['tags'])

    for step in scenario['steps']:
        steps.append(_pickle_step(step))

    pickle = {
        'tags': _pickle_tags(tags),
        'name': scenario['name'],
        'language': language,
        'locations': [_pickle_location(scenario['location'])],
        'steps': steps
    }
    pickles.append(pickle)


def _compile_scenario_outline(feature_tags, background_steps, scenario_outline, language, pickles):
    if len(scenario_outline['steps']) == 0:
        return

    for examples in (e for e in scenario_outline['examples'] if 'tableHeader' in e):
        variable_cells = examples['tableHeader']['cells']

        for values in examples['tableBody']:
            value_cells = values['cells']
            steps = list(background_steps)
            tags = list(feature_tags) \
                + list(scenario_outline['tags']) \
                + list(examples['tags'])

            for scenario_outline_step in scenario_outline['steps']:
                step_text = _interpolate(
                    scenario_outline_step['text'],
                    variable_cells,
                    value_cells)
                arguments = _create_pickle_arguments(
                    scenario_outline_step.get('argument'),
                    variable_cells,
                    value_cells)
                _pickle_step = {
                    'text': step_text,
                    'arguments': arguments,
                    'locations': [
                        _pickle_location(values['location']),
                        _pickle_step_location(scenario_outline_step)
                    ]
                }
                steps.append(_pickle_step)

            pickle = {
                'name': _interpolate(
                    scenario_outline['name'],
                    variable_cells,
                    value_cells),
                'language': language,
                'steps': steps,
                'tags': _pickle_tags(tags),
                'locations': [
                    _pickle_location(values['location']),
                    _pickle_location(scenario_outline['location'])
                ]
            }
            pickles.append(pickle)


def _create_pickle_arguments(argument, variables, values):
    result = []

    if not argument:
        return result

    if argument['type'] is 'DataTable':
        table = {'rows': []}
        for row in argument['rows']:
            cells = [
                {
                    'location': _pickle_location(cell['location']),
                    'value': _interpolate(cell['value'], variables, values)
                } for cell in row['cells']
            ]
            table['rows'].append({'cells': cells})
        result.append(table)

    elif argument['type'] is 'DocString':
        docstring = {
            'location': _pickle_location(argument['location']),
            'content': _interpolate(argument['content'], variables, values)
        }
        result.append(docstring)

    else:
        raise Exception('Internal error')

    return result


def _interpolate(name, variable_cells, value_cells):
    for n, variable_cell in enumerate(variable_cells):
        value_cell = value_cells[n]
        name = re.sub(
            u'<{0[value]}>'.format(variable_cell),
            value_cell['value'],
            name
            )
    return name


def _pickle_steps(scenario_definition):
    return [_pickle_step(step)
            for step in scenario_definition['steps']]


def _pickle_step(step):
    return {
        'text': step['text'],
        'arguments': _create_pickle_arguments(
            step.get('argument'),
            [],
            []),
        'locations': [_pickle_step_location(step)]
    }


def _pickle_step_location(step):
    return {
        'line': step['location']['line'],
        'column': step['location']['column'] + count_symbols(step.get('keyword', 0))
    }


def _pickle_location(location):
    return {
        'line': location['line'],
        'column': location['column']
    }


def _pickle_tags(tags):
    return [_pickle_tag(tag) for tag in tags]


def _pickle_tag(tag):
    return {
        'name': tag['name'],
        'location': _pickle_location(tag['location'])
    }
